#! /usr/bin/python
## -*- coding: utf-8 -*-

##
##  HARK Middleware
##  Copyright (C) 2013-2014 by Honda Research Institute Japan Co., Ltd.
##
##  $Id:$
##

from harkmw.defs.terminal import TerminalDef
from harkmw.defs.parameter import Parameters

class NodeDef :
    """ HARK node in network """

    def __init__(self, network, typedefs) :
        self.network = network
        self.typedefs = typedefs
        self.name    = ''
        self.type    = ''
        self.inputs  = {}
        self.outputs = {}
        self.parameters = Parameters()

        self.process     = None
        self.topic       = None

    def loadXML(self, element) :
        from harkmw.defs.network import NetworkDef

        name = element.attributes['name'].value if element.getAttributeNode('name') else None
        type = element.attributes['type'].value if element.getAttributeNode('type') else None

        if name is None :
            raise HARKError("No attribute 'name' in element <%s>" % element.nodeName)

        if type is None :
            raise HARKError("No attribute 'type' in element <%s>" % element.nodeName)

        # attribute 'x' and 'y' are igonored
        parameters = Parameters()
        parameters.loadXML(element)

        document = self.network.document
        if document is not None :
            typedefs = document.typedefs.typedefs
            typedef  = typedefs[type] if type in typedefs else None
        else :
            typedef = None

        inputs  = {}
        outputs = {}

        if typedef is None :
            # ignore
#           print "[Debug] Typedef of node type '%s' not found" % type
            pass

        else :
            parameters.update(typedef.parameters)

            for (iname, input) in typedef.inputs.items() :
                terminal = TerminalDef(self, input.name, True)
                inputs[iname] = terminal

            for (oname, output) in typedef.outputs.items() :
                terminal = TerminalDef(self, output.name, False)
                outputs[oname] = terminal

#        inputNames = self.network.document.getNetTerminals(NetworkDef.INPUT, type);
#        inputs = {}
#        for i in inputNames :
#            inputs[i] = TerminalDef(self, i, True)
#
#        outputNames = self.network.document.getNetTerminals(NetworkDef.OUTPUT, type);
#        outputs = {}
#        for o in outputNames :
#            outputs[o] = TerminalDef(self, i, False)

        self.name    = name
        self.type    = type
        self.inputs  = inputs
        self.outputs = outputs
        self.parameters = parameters

    def insert_params(self, parameters) :
        self.parameters.update(parameters)

    def findInputTerminal(self, name) :
        from harkmw.defs.network import NetworkDef
        return self.findTerminal(NetworkDef.INPUT, name)

    def findOutputTerminal(self, name) :
        from harkmw.defs.network import NetworkDef
        return self.findTerminal(NetworkDef.OUTPUT, name)

    def findTerminal(self, type, name) :
        from harkmw.defs.network import NetworkDef
        if type == NetworkDef.INPUT :
            return self.inputs[name] if name in self.inputs else None
        elif type == NetworkDef.OUTPUT :
            return self.outputs[name] if name in self.outputs else None
        else :
            return None

    def addInputTerminal(self, name) :
        from harkmw.defs.network import NetworkDef
        return self.addTerminal(NetworkDef.INPUT, name)

    def addOutputTerminal(self, name) :
        from harkmw.defs.network import NetworkDef
        return self.addTerminal(NetworkDef.OUTPUT, name)

    def addTerminal(self, type, name) :
        from harkmw.defs.network import NetworkDef
        from harkmw.defs.terminal import TerminalDef
        if type == NetworkDef.INPUT :
            newTerminal = TerminalDef(self, name, True)
            self.inputs[name] = newTerminal
            return newTerminal

        elif type == NetworkDef.OUTPUT :
            newTerminal = TerminalDef(self, name, False)
            self.outputs[name] = newTerminal
            return newTerminal

        else :
            return None

    def get_process_name(self) :
        if self.process is None :
            return self.network.get_process_name()
        else :
            return self.process

    def dump(self, prefix=''):
        print(prefix + "[Node]")
        print(prefix + "  name=%s" % self.name)
        print(prefix + "  type=%s" % self.type)
        print(prefix + "  process=%s" % self.process)
        print(prefix + "  topic=%s" % self.topic)

        print(prefix + "  [Parameters]")
        self.parameters.dump(prefix + "  ")

# end of file
