#! /usr/bin/python
## -*- coding: utf-8 -*-

##
##  HARK Middleware
##  Copyright (C) 2013-2014 by Honda Research Institute Japan Co., Ltd.
##
##  $Id:$
##

import copy
from harkmw.HarkError import HARKError

class Parameter :
    def __init__(self, name, type, value, description = None) :
        self.name        = name
        self.type        = type
        self.value       = value
        self.description = description
        self.used        = False

    def __str__(self) :
        return "parameter: %s=(%s)%d" % (self.name, self.type, self.value)

class Parameters :
    """
    """

    def __init__(self) :
        self.params = {}

    def clone(self) :
        new_object = Parameters()
        new_object.params = copy.deepcopy(self.params)
        return new_object

    def loadXML(self, node) :
        for child in node.childNodes :
            if child.nodeType == child.ELEMENT_NODE :
                if child.nodeName == 'Parameter' :
                    name = child.attributes['name'].value if child.getAttributeNode('name') else None
                    type = child.attributes['type'].value if child.getAttributeNode('type') else None
                    value = child.attributes['value'].value if child.getAttributeNode('value') else None
                    description = child.attributes['description'].value if child.getAttributeNode('description') else None

#                    print "VALUE=<%s>" % value
                    if not value and name in self.params :
                        value = self.params[name].value
#                    print "VALUE=<%s>, <%s>" % (value, self.params[name]['value'] if name in self.params else None)

                    self.params[name] = Parameter(name, type, value, description)

                elif child.nodeName == 'Comments' :
                    pass

                else :
                    pass
                    #raise HARKError("Unexpected node")

    def add(self, name, type, value, description = None) :
        self.params[name] = Parameter(name, type, value)

    def update(self, default_params) :
#       print "[Before]"
#       print self.params
        for param in default_params.params.values() :
            if param.name not in self.params :
                self.params[param.name] = param
#       print "[After]"
#       print self.params
        return self

    def normalize(self, network_parameters) :
        new_params = {}
        for param in self.params.values() :
            if param.type == 'subnet_param':
                if network_parameters.has_parameter(param.value) :
                    #print "Replace value named %s: from=%s/%s, to=%s/%s" % (param.name, param.type, param.value, network_parameters.get_parameter(param.value).type, network_parameters.get_parameter(param.value).value)
                    new_params[param.name] = network_parameters.get_parameter(param.value)
                else :
                    raise HARKError("Parameter %s has type of %s, but network does not have parameter of the name" % (param.name, param.type))

            else :
                new_params[param.name] = param
        self.params = new_params
        return self

    def has_parameter(self, name) :
        return name in self.params

    def get_parameter(self, name) :
        return self.params[name]

    def dump(self, prefix=''):
        for name, param in self.params.items() :
            print(prefix + '  [%s]: name="%s", type="%s", value="%s", description="%s"' % (name, param.name, param.type, param.value, param.description))

# end of file
