#! /usr/bin/python
## -*- coding: utf-8 -*-

##
##  HARK Middleware
##  Copyright (C) 2013-2014 by Honda Research Institute Japan Co., Ltd.
##
##  $Id:$
##

import threading
import copy

from harkmw.logic.node import Node
from harkmw.logic.link import Link

import os
debug = (bool(os.environ['HARK_DEBUG'])) if 'HARK_DEBUG' in os.environ else False

class Network(Node) :
    def __init__(self) :
        Node.__init__(self);
        self.name       = None
        self.type       = None
        self.network_type       = None
        self.topic      = None
        self.nodes      = {}
        self.links      = []
        self.inputs     = {} # NetTerminal
        self.outputs    = {} # NetTerminal
        self.conditions = {} # NetTerminal
        self.input_terminals = {}
        self.output_terminals = {}
        self.parameters = {}

        self.cv = threading.Condition()
        self.remote_data_offset = 0
        self.remote_data = []
        self.mqtt_proxies = []
        self.mqtt_messages = []

    def run(self, process) :
        """
        """
        if self.process_name == process.process_name :
            self.proceed(0, process)
            self.output_collector.dump_input(0)
        else :
            while True :
                topic, count, data, args, _process = self.pop_mqtt_message()
                if topic is None :
                    process.check_mqtt()
                elif topic == self.get_topic() + "/_END" :
                    break
                else :
                    self.process_mqtt_message(topic, count, data, args, process)

    def proceed(self, count, process) :
        if debug:
            print("Network: proceed: name={}, network_type={}".format(self.name, self.network_type))
        if self.network_type == 'iterator' :
            node = self.nodes['NETWORK_CONDITION']

            for n in self.nodes['ALL_NETWORK_INPUT'].inputs.values() :
                n.force_count(count)

            if self.process_name == process.process_name and process.is_mqtt_initialized():
                pass #process.publish(self.get_topic() + "/_ITERATE", count, True)

            internal_count = 0
            last_count = -1
            while True :
                if debug:
                    print("Loop {}".format(internal_count))

                # ループ終了判定
                if self.process_name == process.process_name :
                    condition_satisfied = node.get_output("OUTPUT", internal_count)
                    if debug:
                        print("condition_satisfied: {}".format(condition_satisfied))
                    if process.is_mqtt_initialized() :
                        pass #process.publish(self.get_topic() + "/_CONDITION", internal_count, condition_satisfied)
                
                if last_count < 0 and not condition_satisfied :
                    last_count = internal_count - 1

                if debug:
                    print("count={}, last_count={}, input_count={}".format(internal_count, last_count, self.nodes['ALL_NETWORK_OUTPUT'].input_count))
                # lookAhead がある場合はもう何フレームが進めることになる。判定は出力段のフレーム番号で行う
                if last_count >= 0:
                    if self.nodes['ALL_NETWORK_OUTPUT'].input_count >= last_count:
                        if debug:
                            print("breaking loop")
                        break

                self.proceed_frame(internal_count, process, outer_count=count)
                internal_count += 1

            if self.process_name == process.process_name and process.is_mqtt_initialized() :
                pass #process.publish(self.get_topic() + "/_ITERATE", count, False)

        else :
            count = self.proceed_frame(count, process)

        if count >= 0:
            for (output_name, output) in self.nodes['ALL_NETWORK_OUTPUT'].outputs.items() :
                for output_proxy in output :
                    #print("Network: Node.push_data: Push: %s: %d: %s" % (self.name, count, output_proxy))
                    if output_proxy is not None :
                        output_proxy.push_data(count, process)
        return count
                
    def proceed_frame(self, count, process, outer_count=-1) :
        if debug:
            print("Proceed frame: {}, count={}".format(self.name, count))
        if outer_count < 0:
            outer_count = count

        need_publish = self.has_remote_nodes(process)

        if self.process_name == process.process_name and process.is_mqtt_initialized() :
            if need_publish :
                process.publish(self.get_topic() + "/_START", count, True)

        minimum_count = count
        if debug:
            for node in self.nodes.values() :
                print("node: name={}, type={}, is-input={}, is-network={}, input-count={}".format(node.name, node.type, (node.name[-6:] == "_input"), (isinstance(node, Network)), (node.get_input_count() == 0)))

        for node in self.nodes.values() :
            if not isinstance(node, Network) and node.name[-6:] != '_input' and node.get_input_count() == 0 :
                if debug:
                    print("calling {} as spring node".format(node.name))
                minimum_count = min(minimum_count, node.push_data(None, count, process))

        for node in self.nodes.values() :
            if node.name[-6:] == "_input" :
                # this node is network-input collector
                if debug:
                    print("calling {} as network-input collector".format(node.name))
                minimum_count = min(minimum_count, node.push_data(None, count, process, outer_count=outer_count))

        for node in self.nodes.values() :
            if isinstance(node, Network) and len(node.inputs) == 0:
                if debug:
                    print("calling {} as spring network".format(node.name))
                    print("node: {}, inputs={}, outputs={}".format(node, node.inputs, node.outputs))
                minimum_count = min(minimum_count, node.push_data(None, count, process))

        mqtt_proxies = copy.copy(self.mqtt_proxies)
        while len(mqtt_proxies) > 0:
            while not self.has_remote_data(count) :
                process.check_mqtt()

            data = self.get_remote_data(count)
            for d in data:
                proxy = d['proxy']
                proxy.push_data(count, process, d['data'])
                mqtt_proxies.remove(proxy)

        if self.process_name == process.process_name and process.is_mqtt_initialized() :
            if need_publish :
                process.publish(self.get_topic() + "/_END", count, True)

        if debug:
            print("Return from proceed frame: {}, count={}, minimum_count={}".format(self.name, count, minimum_count))
        return minimum_count

    def request(self, process) :
        for output in [self.nodes['ALL_NETWORK_OUTPUT']]:
            output.request(process)
    
    def initialize(self, process) :
        node_counts = {}
        for node in self.nodes.values() :
            node.initialize(process)
            node_counts[node] = 0

        for input in self.inputs.values() :
            input.initialize(process)

        self.node_counts = node_counts # ノードあたりの最新 count 数
        self.count       = 0           # ネットワーク内の最大 count 数

        if self.process_name != process.process_name :
            if self.has_local_nodes(process) :
                receiver = self.parent_network if self.parent_network is not None else self
                process.subscribe(self.get_topic() + "/_START",     receiver, self)
                process.subscribe(self.get_topic() + "/_END",       receiver, self)
                process.subscribe(self.get_topic() + "/_ITERATE",   receiver, self)
                process.subscribe(self.get_topic() + "/_CONDITION", receiver, self)

    def release(self) :
        for node in self.nodes.values() :
            node.release()
        self.nodes = {}
        self.inputs = {}
        self.outputs = {}
        self.conditions = {}
        self.links = []
        self.count = 0

    def has_local_nodes(self, process) :
        for n in self.nodes.values() :
            if isinstance(n, Network) :
                if n.has_local_nodes(process) :
                    return True
            elif n.process_name == process.process_name :
                return True
        return False

    def has_remote_nodes(self, process) :
        for n in self.nodes.values() :
            if isinstance(n, Network) :
                if n.has_remote_nodes(process) :
                    return True
            if n.process_name != process.process_name :
                return True
        return False

    def push_data(self, input, count, process) :
        """ ループ count のデータが input に入力された場合にコールされる
        処理されたノードの集合を返す
        """
        return self.proceed(count, process)

    def pull_data(self, output, count) :
        """
        """
        data = None
        for output in self.outputs.values() :
            output.pull_data(self, count)

        return data

    def connect_nodes(self, node_from, terminal_from, node_to, terminal_to) :
        node_from.connect_node(terminal_from, node_to, terminal_to)

    def get_input_count(self) :
        return len(self.inputs)

    def get_output_count(self) :
        return len(self.outputs)

    def get_topic(self) :
        if self.topic is None :
            if self.network is None :
                return "%s" % self.name
            else :
                return "%s/%s" % (self.network.get_topic(), self.name)
        else :
            return self.topic

    def process_mqtt_message(self, topic, count, data, args, process) :
        if topic == args.get_topic() + "/_START" :
            args.proceed_frame(count, process)

        elif topic == self.get_topic() + "/_END" :
            pass

        elif topic == self.get_topic() + "/_ITERATE" :
            pass

        elif topic == self.get_topic() + "/_CONDITION" :
            pass

        else :
            pass

    def add_mqtt_message(self, topic, count, data, args, process) :
        self.mqtt_messages.append((topic, count, data, args, process))

    def pop_mqtt_message(self) :
        if len(self.mqtt_messages) > 0 :
            return self.mqtt_messages.pop()
        return (None, None, None, None, None)

#    def wait_mqtt_message(self, topic, count, process) :
#        while topic not in self.mqtt_messages or count not in self.mqtt_messages[topic] :
#            process.check_mqtt()
#        return self.mqtt_messages[topic].pop(count)

    def add_remote_data(self, count, proxy, args, data) :
        while self.remote_data_offset > count or count >= self.remote_data_offset + len(self.remote_data) :
            self.remote_data.append([])

        self.remote_data[count - self.remote_data_offset].append({'proxy': proxy, 'data': data})

    def get_remote_data(self, count) :
        if self.remote_data_offset <= count and count < self.remote_data_offset + len(self.remote_data) :
            data = self.remote_data[count - self.remote_data_offset]
            self.remote_data[count - self.remote_data_offset] = []
            return data
        else :
            return []

    def has_remote_data(self, count) :
        if self.remote_data_offset <= count and count < self.remote_data_offset + len(self.remote_data) :
            return len(self.remote_data[count - self.remote_data_offset]) > 0
        else :
            return False

    def dump(self) :
        print('[Network]')
        print('  name: %s' % self.name)
        print('  type: %s' % self.type)

        print('  inputs: ')
        for (name, terminal) in self.inputs.items() :
            print('    Input: name=%s, node=%s, node-terminal=%s' % (name, terminal.get_input_node().name if terminal.get_input_node() is not None else "(null)", terminal.get_input_terminal_name()))

        print('  outputs: ')
        for (name, oo) in self.outputs.items() :
            for output in oo :
                print('    Output: name=%s, node=%s, node-terminal=%s' % (name, terminal.get_output_node().name if terminal.get_output_node() is not None else "(null)", terminal.get_output_terminal_name()))

        print('  conditions: ')
        for (name, terminal) in self.conditions.items() :
            print('    Condition: name=%s, node=%s, node-terminal=%s' % (name, terminal.node.name, terminal.name))

        print('  nodes: ')
        for (name, node) in self.nodes.items() :
            node.dump()

    def __str__(self) :
        return "<Network: type=%s, name=%s>" % (self.type, self.name)

# end of file
