#! /usr/bin/python
## -*- coding: utf-8 -*-

##
##  HARK Middleware main
##  Copyright (C) 2018 by Honda Research Institute Japan Co., Ltd.
##
##  $Id:$
##

import sys, getopt
import os
import pkg_resources
import argparse
import harkmw

def version(dirs) :
    app = harkmw.Process("")
    app.print_modules(dirs)

def usage() :
    print("""
Usage:
    $ hark-middleware [-x/--execute FILE]
             [-t/--toolbox DIR[:DIR[...]]] [-p/--process NAME]
             [-v/--version] [-V/--mwversion] [-h/--help] NETWORK [args ...]

Options:
    NETWORK         Load .n file from NETWORK.

    -x FILE         Load .x file from FILE.
    --execute FILE  If omitted, .n file is used.

    -t DIR          Load toolbox from DIR.
    --toolbox DIR   If omitted, default value
                    Ubuntu  /usr/local/lib/hark/toolbox:/usr/lib/hark/toolbox
                    Windows C:/Program Files/HARK/lib/hark/toolbox
                    is used.

    -p NAME         Run as NAME.
    --process NAME  If omitted, default value 'local' is used.

    -v              Show module version information and exit.
    --version

    -V              Show hark-middleware version information and exit.
    --mwversion

    -h              Show help information and exit.
    --help

""")

if os.name =='nt': default_toolbox_dir = os.getenv('HARK_PATH') if os.getenv('HARK_PATH') else 'C:/Program Files/HARK/lib/hark/toolbox'
else: default_toolbox_dir = os.getenv('HARK_PATH') if os.getenv('HARK_PATH') else '/usr/local/lib/hark/toolbox:/usr/lib/hark/toolbox'
default_process_name = 'local'

def main() :
    parser = argparse.ArgumentParser(description='HARK middleware')
    parser.add_argument('--execute', '-x', metavar='FILE', type=str, help='Load .x file from FILE.  If omitted, .n file is used.', default=None)
    parser.add_argument('--toolbox', '-t', metavar='DIR',  type=str, help='Load toolbox from DIR.  If omitted, default value %(default)s is used.', default=default_toolbox_dir)
    parser.add_argument('--process', '-p', metavar='NAME', type=str, help='Run as NAME.  If omitted, default value \'%(default)s\' is used.', default=default_process_name)
    parser.add_argument('--version', '-v', action='store_true', help='Show module version information and exit.', default=False)
    parser.add_argument('--mwversion', '-V', action='version', help='Show hark-middleware version information and exit.', version='HARK middleware {}'.format(pkg_resources.get_distribution('harkmw').version))
    parser.add_argument('network', nargs='?', help='Load .n file from NETWORK.')
    parser.add_argument('arg', nargs='*', help='Arguments to be passed to network.')

    args = parser.parse_args()
    #print(args)

    # Set option values
    network_file = args.network
    execute_file = args.execute
    if os.name =='nt': toolbox_dirs = args.toolbox.split(";")
    else: toolbox_dirs = args.toolbox.split(":")
    if args.version:
        version(toolbox_dirs)
        sys.exit(0)
    process_name = args.process
    args         = args.arg

    if execute_file is None :
        execute_file = network_file

    if network_file is None and execute_file is None :
        parser.print_help(sys.stderr)
        sys.exit(1)

    # Dump options
    print("Start HARK middleware:")
    print("    network: %s" % network_file)
    print("    execute: %s" % execute_file)
    print("    process: %s" % process_name)
    print("    toolbox: %s" % toolbox_dirs)
    print("    args:    %s" % args)
    sys.stdout.flush()

    # Create application instance
    app = harkmw.Process(args)
    app.add_toolbox_dirs(toolbox_dirs)
    app.load(network_file, execute_file)
    app.run(process_name)

if __name__ == '__main__' :
    main()

# end of file
