#! /usr/bin/python
## -*- coding: utf-8 -*-

##
##  HARK Middleware
##  Copyright (C) 2013-2014 by Honda Research Institute Japan Co., Ltd.
##
##  $Id:$
##

from harkmw.HarkError import HARKError
from harkmw.defs.node import NodeDef
from harkmw.defs.parameter import Parameters

import harkmw.logic.node
import harkmwnative
from collections import defaultdict

class NativeNode(harkmw.logic.node.Node) :
    def __init__(self) :
        harkmw.logic.node.Node.__init__(self)
        self.inputs  = {}
        self.outputs = {}
        self.network = None
        self.count   = -1
        self.input_count  = -1
        self.output_count = -1
        self.input_look_windows  = defaultdict(lambda: (0, 0))
        self.output_look_windows = defaultdict(lambda: (0, 0))

    def build(self, network, node_def, params) :
        self.network     = network
        self.type        = node_def.type
        self.name        = node_def.name
        self.topic       = node_def.topic
        self.process_name = node_def.get_process_name()

#       print "NativeNode.build: params: "
#       params.dump()
#       print "NativeNode.build: node_def.parameters: "
#       node_def.parameters.dump()

        # Load native module
        self.native_node = harkmwnative.Node(self, node_def.type, node_def.parameters)
#       try :
            #print "normalize params: %s" % self.name
            #node_def.parameters.normalize(params)
            #print "Params:"
            #params.dump()
            #print "node_def.parameters"
            #node_def.parameters.dump()
#           self.native_nnode = harkmwnative.Node(self, node_def.type, node_def.parameters)

#       except NameError, e :
#           raise HARKError("Failed to create node with name '%s'" % node_def.type)

        # Setup node defs
#         inputs = {}
#         for input in node_def.inputs.values() :
#             input = Terminal(self, input.name, True, 1)
#             inputs[input.name] = input
# 
#         for output in node_def.outputs.values() :
#             output = Terminal(self, output.name, False, 1)
#             self.outputs[output.name] = output
# 
#         self.inputs      = inputs

    def build_type(self, network, type, name = None) :
        # Load native module
        self.native_node = harkmwnative.Node(self, type, Parameters())
#       try :
#           self.native_node = harkmwnative.Node(self, type, Parameters())
#       except NameError :
#           raise HARKError("Failed to create node with name '%s'" % type)

        self.network     = network
        self.type        = type
        self.name        = name
        self.topic       = None
        self.inputs      = {}
        self.outputs     = {}
        self.process_name = network.process_name

    def initialize(self, process) :
        #print "NativeNode.initialize(): %s" % self.inputs
        self.native_node.initialize()
        harkmw.logic.node.Node.initialize(self, process)
        #for input in self.inputs.values() :
        #    input.initialize()

    def release(self) :
        self.native_node = None
        self.inputs = {}
        self.outputs = {}

    def process(self, output, count, input_count=-1, output_count=-1):
        #print "NativeNode: process: count=%d, input-count=%d, output-count=%d" % (count, input_count, output_count)
        self.native_node.process(output, count, input_count=input_count, output_count=output_count)
        self.count = count

    def set_output_look_window(self, output_id, look_window):
        self.output_look_windows[output_id] = look_window
        return self.native_node.set_output_look_window(output_id, look_window[0], look_window[1])

    def get_output_look_window(self, output_id):
        look_window = self.native_node.get_output_look_window(output_id)
        return max(look_window, self.output_look_windows[output_id])

    def set_input_look_window(self, input_id, look_window):
        self.input_look_windows[output_id] = look_window
        return self.native_node.set_input_look_window(input_id, look_window[0], look_window[1])

    def get_input_look_window(self, input_id):
        look_window = self.native_node.get_input_look_window(input_id)
        return max(look_window, self.input_look_windows[input_id])

    def get_input_count(self) :
        return self.native_node.get_input_count()

    def get_output_count(self) :
        return self.native_node.get_output_count()

    def get_output(self, name, count) :
#       print "NativeNode: get_output: %d" % count
        return self.native_node.get_output(name, count)

    def dump_input(self, count):
        return self.native_node.dump_input(count)

    def connect_node(self, terminal_name, node_from, terminal_from_name) :
        #print "TerminalName: %s,  NodeFrom: %s,  TerminalFromName: %s" % (terminal_name, node_from, terminal_from_name)
        self.native_node.connect(terminal_name, node_from.native_node, terminal_from_name)

    def dump(self) :
        print('[NativeNode]')
        print('  type: %s' % self.type)
        print('  name: %s' % self.name)
        print('  network: type=%s, name=%s' % (self.network.type, self.network.name))

        print('  inputs: ')
        for (name, terminal) in self.inputs.items() :
            print('    Input: name=%s, node=%s/%s, terminal=%s' % (name, terminal.input_node.type, terminal.input_node.name, terminal.input_terminal_name))

        print('  outputs: ')
        for (name, proxy_list) in self.outputs.items() :
            for proxy in proxy_list :
                print('    Output: name=%s, node=%s/%s, terminal=%s' % (name, proxy.output_node.type, proxy.output_node.name, proxy.output_terminal_name))

    def __str__(self) :
        return "<%s: type=%s, name=%s>" % (self.__class__.__name__, self.type, self.name)

# end of file
